# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './caiman_motion_correction.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(287, 842)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_motion_correction_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_motion_correction_kwargs.setStatusTip("")
        self.groupBox_motion_correction_kwargs.setCheckable(True)
        self.groupBox_motion_correction_kwargs.setChecked(False)
        self.groupBox_motion_correction_kwargs.setObjectName("groupBox_motion_correction_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_motion_correction_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_mc_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_motion_correction_kwargs)
        self.plainTextEdit_mc_kwargs.setObjectName("plainTextEdit_mc_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_mc_kwargs)
        self.verticalLayout.addWidget(self.groupBox_motion_correction_kwargs)
        self.label_16 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.verticalLayout.addWidget(self.label_16)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_11 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_11.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_21.addWidget(self.label_11)
        self.comboBoxOutputBitDepth = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxOutputBitDepth.setObjectName("comboBoxOutputBitDepth")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.horizontalLayout_21.addWidget(self.comboBoxOutputBitDepth)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_21)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxGSig_filt = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGSig_filt.setObjectName("spinBoxGSig_filt")
        self.horizontalLayout.addWidget(self.spinBoxGSig_filt)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_12.addWidget(self.label_9)
        self.spinboxX = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxX.sizePolicy().hasHeightForWidth())
        self.spinboxX.setSizePolicy(sizePolicy)
        self.spinboxX.setMaximum(999)
        self.spinboxX.setObjectName("spinboxX")
        self.horizontalLayout_12.addWidget(self.spinboxX)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_13 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_13.addWidget(self.label_13)
        self.spinboxY = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxY.sizePolicy().hasHeightForWidth())
        self.spinboxY.setSizePolicy(sizePolicy)
        self.spinboxY.setMaximum(999)
        self.spinboxY.setObjectName("spinboxY")
        self.horizontalLayout_13.addWidget(self.spinboxY)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_14.addWidget(self.label_6)
        self.spinboxIterRigid = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxIterRigid.sizePolicy().hasHeightForWidth())
        self.spinboxIterRigid.setSizePolicy(sizePolicy)
        self.spinboxIterRigid.setMinimum(1)
        self.spinboxIterRigid.setMaximum(5)
        self.spinboxIterRigid.setObjectName("spinboxIterRigid")
        self.horizontalLayout_14.addWidget(self.spinboxIterRigid)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.label_17 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.verticalLayout.addWidget(self.label_17)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.label_15 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_15.setAlignment(QtCore.Qt.AlignCenter)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_16.addWidget(self.label_15)
        self.spinboxMaxDev = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxMaxDev.setMaximum(999)
        self.spinboxMaxDev.setProperty("value", 3)
        self.spinboxMaxDev.setObjectName("spinboxMaxDev")
        self.horizontalLayout_16.addWidget(self.spinboxMaxDev)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_16)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_18 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_18.sizePolicy().hasHeightForWidth())
        self.label_18.setSizePolicy(sizePolicy)
        self.label_18.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_17.addWidget(self.label_18)
        self.spinboxStrides = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxStrides.setMaximum(500)
        self.spinboxStrides.setProperty("value", 0)
        self.spinboxStrides.setObjectName("spinboxStrides")
        self.horizontalLayout_17.addWidget(self.spinboxStrides)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem6)
        self.verticalLayout_5.addLayout(self.horizontalLayout_17)
        self.sliderStrides = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderStrides.setMinimum(1)
        self.sliderStrides.setMaximum(500)
        self.sliderStrides.setOrientation(QtCore.Qt.Horizontal)
        self.sliderStrides.setObjectName("sliderStrides")
        self.verticalLayout_5.addWidget(self.sliderStrides)
        self.verticalLayout.addLayout(self.verticalLayout_5)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_19 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_19.sizePolicy().hasHeightForWidth())
        self.label_19.setSizePolicy(sizePolicy)
        self.label_19.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_18.addWidget(self.label_19)
        self.spinboxOverlaps = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxOverlaps.setMaximum(500)
        self.spinboxOverlaps.setProperty("value", 0)
        self.spinboxOverlaps.setObjectName("spinboxOverlaps")
        self.horizontalLayout_18.addWidget(self.spinboxOverlaps)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem7)
        self.verticalLayout_6.addLayout(self.horizontalLayout_18)
        self.sliderOverlaps = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderOverlaps.setMinimum(1)
        self.sliderOverlaps.setMaximum(500)
        self.sliderOverlaps.setOrientation(QtCore.Qt.Horizontal)
        self.sliderOverlaps.setObjectName("sliderOverlaps")
        self.verticalLayout_6.addWidget(self.sliderOverlaps)
        self.verticalLayout.addLayout(self.verticalLayout_6)
        self.btnShowQuilt = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnShowQuilt.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnShowQuilt.setCheckable(True)
        self.btnShowQuilt.setObjectName("btnShowQuilt")
        self.verticalLayout.addWidget(self.btnShowQuilt)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_10 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_19.addWidget(self.label_10)
        self.spinboxUpsample = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxUpsample.setMinimum(1)
        self.spinboxUpsample.setMaximum(10)
        self.spinboxUpsample.setProperty("value", 4)
        self.spinboxUpsample.setObjectName("spinboxUpsample")
        self.horizontalLayout_19.addWidget(self.spinboxUpsample)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_19)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.lineEditNameElastic = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEditNameElastic.setObjectName("lineEditNameElastic")
        self.horizontalLayout_20.addWidget(self.lineEditNameElastic)
        self.btnAddToBatchElastic = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchElastic.setObjectName("btnAddToBatchElastic")
        self.horizontalLayout_20.addWidget(self.btnAddToBatchElastic)
        self.verticalLayout.addLayout(self.horizontalLayout_20)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.comboBoxProjectionsOption = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxProjectionsOption.setObjectName("comboBoxProjectionsOption")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.horizontalLayout_2.addWidget(self.comboBoxProjectionsOption)
        self.pushButtonViewProjections = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonViewProjections.setObjectName("pushButtonViewProjections")
        self.horizontalLayout_2.addWidget(self.pushButtonViewProjections)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.sliderOverlaps.valueChanged['int'].connect(self.spinboxOverlaps.setValue)
        self.sliderStrides.valueChanged['int'].connect(self.spinboxStrides.setValue)
        self.spinboxOverlaps.valueChanged['int'].connect(self.sliderOverlaps.setValue)
        self.spinboxStrides.valueChanged['int'].connect(self.sliderStrides.setValue)
        self.lineEditNameElastic.returnPressed.connect(self.btnAddToBatchElastic.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.groupBox_motion_correction_kwargs, self.plainTextEdit_mc_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_mc_kwargs, self.comboBoxOutputBitDepth)
        DockWidget.setTabOrder(self.comboBoxOutputBitDepth, self.spinBoxGSig_filt)
        DockWidget.setTabOrder(self.spinBoxGSig_filt, self.spinboxX)
        DockWidget.setTabOrder(self.spinboxX, self.spinboxY)
        DockWidget.setTabOrder(self.spinboxY, self.spinboxIterRigid)
        DockWidget.setTabOrder(self.spinboxIterRigid, self.spinboxMaxDev)
        DockWidget.setTabOrder(self.spinboxMaxDev, self.spinboxStrides)
        DockWidget.setTabOrder(self.spinboxStrides, self.sliderStrides)
        DockWidget.setTabOrder(self.sliderStrides, self.spinboxOverlaps)
        DockWidget.setTabOrder(self.spinboxOverlaps, self.sliderOverlaps)
        DockWidget.setTabOrder(self.sliderOverlaps, self.btnShowQuilt)
        DockWidget.setTabOrder(self.btnShowQuilt, self.spinboxUpsample)
        DockWidget.setTabOrder(self.spinboxUpsample, self.lineEditNameElastic)
        DockWidget.setTabOrder(self.lineEditNameElastic, self.btnAddToBatchElastic)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&CaImAn Motion Correction"))
        self.groupBox_motion_correction_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for MotionCorrect() instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_motion_correction_kwargs.setTitle(_translate("DockWidget", "Use &motion correction kwargs"))
        self.plainTextEdit_mc_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_16.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_16.setText(_translate("DockWidget", "Rigid correction"))
        self.label_11.setText(_translate("DockWidget", "Output bit depth: "))
        self.comboBoxOutputBitDepth.setItemText(0, _translate("DockWidget", "Do not convert"))
        self.comboBoxOutputBitDepth.setItemText(1, _translate("DockWidget", "8"))
        self.comboBoxOutputBitDepth.setItemText(2, _translate("DockWidget", "16"))
        self.label.setText(_translate("DockWidget", "gSig_filt:"))
        self.spinBoxGSig_filt.setToolTip(_translate("DockWidget", "disabled if set to zero"))
        self.label_9.setText(_translate("DockWidget", " max shifts X (pixels): "))
        self.label_13.setText(_translate("DockWidget", " max shifts Y (pixels): "))
        self.label_6.setText(_translate("DockWidget", " iterations for rigid: "))
        self.label_17.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_17.setText(_translate("DockWidget", "Elastic correction"))
        self.label_15.setText(_translate("DockWidget", " max deviation from rigid:"))
        self.label_18.setText(_translate("DockWidget", "strides (pixels): "))
        self.label_19.setText(_translate("DockWidget", "overlaps (pixels): "))
        self.btnShowQuilt.setText(_translate("DockWidget", "Show Grid"))
        self.label_10.setText(_translate("DockWidget", " upsample grid:"))
        self.lineEditNameElastic.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchElastic.setText(_translate("DockWidget", "Add to batch"))
        self.label_2.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.label_2.setText(_translate("DockWidget", "View projections of selected outputs"))
        self.comboBoxProjectionsOption.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.comboBoxProjectionsOption.setItemText(0, _translate("DockWidget", "mean"))
        self.comboBoxProjectionsOption.setItemText(1, _translate("DockWidget", "max"))
        self.comboBoxProjectionsOption.setItemText(2, _translate("DockWidget", "std"))
        self.pushButtonViewProjections.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.pushButtonViewProjections.setText(_translate("DockWidget", "View"))


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './caiman_motion_correction.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(279, 869)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_motion_correction_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_motion_correction_kwargs.setStatusTip("")
        self.groupBox_motion_correction_kwargs.setCheckable(True)
        self.groupBox_motion_correction_kwargs.setChecked(False)
        self.groupBox_motion_correction_kwargs.setObjectName("groupBox_motion_correction_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_motion_correction_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_mc_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_motion_correction_kwargs)
        self.plainTextEdit_mc_kwargs.setObjectName("plainTextEdit_mc_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_mc_kwargs)
        self.verticalLayout.addWidget(self.groupBox_motion_correction_kwargs)
        self.label_16 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.verticalLayout.addWidget(self.label_16)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_11 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_11.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_21.addWidget(self.label_11)
        self.comboBoxOutputBitDepth = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxOutputBitDepth.setObjectName("comboBoxOutputBitDepth")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.comboBoxOutputBitDepth.addItem("")
        self.horizontalLayout_21.addWidget(self.comboBoxOutputBitDepth)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_21)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxGSig_filt = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGSig_filt.setObjectName("spinBoxGSig_filt")
        self.horizontalLayout.addWidget(self.spinBoxGSig_filt)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_9.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_12.addWidget(self.label_9)
        self.spinboxX = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxX.sizePolicy().hasHeightForWidth())
        self.spinboxX.setSizePolicy(sizePolicy)
        self.spinboxX.setMaximum(999)
        self.spinboxX.setObjectName("spinboxX")
        self.horizontalLayout_12.addWidget(self.spinboxX)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_13 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_13.addWidget(self.label_13)
        self.spinboxY = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxY.sizePolicy().hasHeightForWidth())
        self.spinboxY.setSizePolicy(sizePolicy)
        self.spinboxY.setMaximum(999)
        self.spinboxY.setObjectName("spinboxY")
        self.horizontalLayout_13.addWidget(self.spinboxY)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_14.addWidget(self.label_6)
        self.spinboxIterRigid = QtWidgets.QSpinBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinboxIterRigid.sizePolicy().hasHeightForWidth())
        self.spinboxIterRigid.setSizePolicy(sizePolicy)
        self.spinboxIterRigid.setMinimum(1)
        self.spinboxIterRigid.setMaximum(5)
        self.spinboxIterRigid.setObjectName("spinboxIterRigid")
        self.horizontalLayout_14.addWidget(self.spinboxIterRigid)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.label_17 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.verticalLayout.addWidget(self.label_17)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.label_15 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_15.setAlignment(QtCore.Qt.AlignCenter)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_16.addWidget(self.label_15)
        self.spinboxMaxDev = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxMaxDev.setMaximum(999)
        self.spinboxMaxDev.setProperty("value", 3)
        self.spinboxMaxDev.setObjectName("spinboxMaxDev")
        self.horizontalLayout_16.addWidget(self.spinboxMaxDev)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_16)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_18 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_18.sizePolicy().hasHeightForWidth())
        self.label_18.setSizePolicy(sizePolicy)
        self.label_18.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_17.addWidget(self.label_18)
        self.spinboxStrides = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxStrides.setMaximum(500)
        self.spinboxStrides.setProperty("value", 0)
        self.spinboxStrides.setObjectName("spinboxStrides")
        self.horizontalLayout_17.addWidget(self.spinboxStrides)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem6)
        self.verticalLayout_5.addLayout(self.horizontalLayout_17)
        self.sliderStrides = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderStrides.setMinimum(1)
        self.sliderStrides.setMaximum(500)
        self.sliderStrides.setOrientation(QtCore.Qt.Horizontal)
        self.sliderStrides.setObjectName("sliderStrides")
        self.verticalLayout_5.addWidget(self.sliderStrides)
        self.verticalLayout.addLayout(self.verticalLayout_5)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_19 = QtWidgets.QLabel(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_19.sizePolicy().hasHeightForWidth())
        self.label_19.setSizePolicy(sizePolicy)
        self.label_19.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_18.addWidget(self.label_19)
        self.spinboxOverlaps = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxOverlaps.setMaximum(500)
        self.spinboxOverlaps.setProperty("value", 0)
        self.spinboxOverlaps.setObjectName("spinboxOverlaps")
        self.horizontalLayout_18.addWidget(self.spinboxOverlaps)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem7)
        self.verticalLayout_6.addLayout(self.horizontalLayout_18)
        self.sliderOverlaps = QtWidgets.QSlider(self.dockWidgetContents)
        self.sliderOverlaps.setMinimum(1)
        self.sliderOverlaps.setMaximum(500)
        self.sliderOverlaps.setOrientation(QtCore.Qt.Horizontal)
        self.sliderOverlaps.setObjectName("sliderOverlaps")
        self.verticalLayout_6.addWidget(self.sliderOverlaps)
        self.verticalLayout.addLayout(self.verticalLayout_6)
        self.btnShowQuilt = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnShowQuilt.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnShowQuilt.setCheckable(True)
        self.btnShowQuilt.setObjectName("btnShowQuilt")
        self.verticalLayout.addWidget(self.btnShowQuilt)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_10 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_19.addWidget(self.label_10)
        self.spinboxUpsample = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinboxUpsample.setMinimum(1)
        self.spinboxUpsample.setMaximum(10)
        self.spinboxUpsample.setProperty("value", 4)
        self.spinboxUpsample.setObjectName("spinboxUpsample")
        self.horizontalLayout_19.addWidget(self.spinboxUpsample)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_19)
        self.checkBoxKeepMemmap = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxKeepMemmap.setObjectName("checkBoxKeepMemmap")
        self.verticalLayout.addWidget(self.checkBoxKeepMemmap)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.lineEditNameElastic = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEditNameElastic.setObjectName("lineEditNameElastic")
        self.horizontalLayout_20.addWidget(self.lineEditNameElastic)
        self.btnAddToBatchElastic = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchElastic.setObjectName("btnAddToBatchElastic")
        self.horizontalLayout_20.addWidget(self.btnAddToBatchElastic)
        self.verticalLayout.addLayout(self.horizontalLayout_20)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.comboBoxProjectionsOption = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxProjectionsOption.setObjectName("comboBoxProjectionsOption")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.comboBoxProjectionsOption.addItem("")
        self.horizontalLayout_2.addWidget(self.comboBoxProjectionsOption)
        self.pushButtonViewProjections = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonViewProjections.setObjectName("pushButtonViewProjections")
        self.horizontalLayout_2.addWidget(self.pushButtonViewProjections)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.sliderOverlaps.valueChanged['int'].connect(self.spinboxOverlaps.setValue)
        self.sliderStrides.valueChanged['int'].connect(self.spinboxStrides.setValue)
        self.spinboxOverlaps.valueChanged['int'].connect(self.sliderOverlaps.setValue)
        self.spinboxStrides.valueChanged['int'].connect(self.sliderStrides.setValue)
        self.lineEditNameElastic.returnPressed.connect(self.btnAddToBatchElastic.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.groupBox_motion_correction_kwargs, self.plainTextEdit_mc_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_mc_kwargs, self.comboBoxOutputBitDepth)
        DockWidget.setTabOrder(self.comboBoxOutputBitDepth, self.spinBoxGSig_filt)
        DockWidget.setTabOrder(self.spinBoxGSig_filt, self.spinboxX)
        DockWidget.setTabOrder(self.spinboxX, self.spinboxY)
        DockWidget.setTabOrder(self.spinboxY, self.spinboxIterRigid)
        DockWidget.setTabOrder(self.spinboxIterRigid, self.spinboxMaxDev)
        DockWidget.setTabOrder(self.spinboxMaxDev, self.spinboxStrides)
        DockWidget.setTabOrder(self.spinboxStrides, self.sliderStrides)
        DockWidget.setTabOrder(self.sliderStrides, self.spinboxOverlaps)
        DockWidget.setTabOrder(self.spinboxOverlaps, self.sliderOverlaps)
        DockWidget.setTabOrder(self.sliderOverlaps, self.btnShowQuilt)
        DockWidget.setTabOrder(self.btnShowQuilt, self.spinboxUpsample)
        DockWidget.setTabOrder(self.spinboxUpsample, self.checkBoxKeepMemmap)
        DockWidget.setTabOrder(self.checkBoxKeepMemmap, self.lineEditNameElastic)
        DockWidget.setTabOrder(self.lineEditNameElastic, self.btnAddToBatchElastic)
        DockWidget.setTabOrder(self.btnAddToBatchElastic, self.comboBoxProjectionsOption)
        DockWidget.setTabOrder(self.comboBoxProjectionsOption, self.pushButtonViewProjections)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&CaImAn Motion Correction"))
        self.groupBox_motion_correction_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for MotionCorrect() instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_motion_correction_kwargs.setTitle(_translate("DockWidget", "Use &motion correction kwargs"))
        self.plainTextEdit_mc_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_16.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_16.setText(_translate("DockWidget", "Rigid correction"))
        self.label_11.setText(_translate("DockWidget", "Output bit depth: "))
        self.comboBoxOutputBitDepth.setItemText(0, _translate("DockWidget", "Do not convert"))
        self.comboBoxOutputBitDepth.setItemText(1, _translate("DockWidget", "8"))
        self.comboBoxOutputBitDepth.setItemText(2, _translate("DockWidget", "16"))
        self.label.setText(_translate("DockWidget", "gSig_filt:"))
        self.spinBoxGSig_filt.setToolTip(_translate("DockWidget", "disabled if set to zero"))
        self.label_9.setText(_translate("DockWidget", " max shifts X (pixels): "))
        self.label_13.setText(_translate("DockWidget", " max shifts Y (pixels): "))
        self.label_6.setText(_translate("DockWidget", " iterations for rigid: "))
        self.label_17.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_17.setText(_translate("DockWidget", "Elastic correction"))
        self.label_15.setText(_translate("DockWidget", " max deviation from rigid:"))
        self.label_18.setText(_translate("DockWidget", "strides (pixels): "))
        self.label_19.setText(_translate("DockWidget", "overlaps (pixels): "))
        self.btnShowQuilt.setText(_translate("DockWidget", "Show Grid"))
        self.label_10.setText(_translate("DockWidget", " upsample grid:"))
        self.checkBoxKeepMemmap.setText(_translate("DockWidget", "Keep memmap"))
        self.lineEditNameElastic.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchElastic.setText(_translate("DockWidget", "Add to batch"))
        self.label_2.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.label_2.setText(_translate("DockWidget", "View projections of selected outputs"))
        self.comboBoxProjectionsOption.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.comboBoxProjectionsOption.setItemText(0, _translate("DockWidget", "mean"))
        self.comboBoxProjectionsOption.setItemText(1, _translate("DockWidget", "max"))
        self.comboBoxProjectionsOption.setItemText(2, _translate("DockWidget", "std"))
        self.pushButtonViewProjections.setToolTip(_translate("DockWidget", "View Mean, Max or Std projections of the outputs from selected batch items"))
        self.pushButtonViewProjections.setText(_translate("DockWidget", "View"))





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 18 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

"""
from PyQt5 import QtCore, QtGui, QtWidgets
from .add_to_project_dialog_pytemplate import Ui_Form
from ...common import configuration, get_project_manager
from .viewer_work_environment import ViewerWorkEnv
from numpy import int64, float64
import traceback


class AddToProjectDialog(QtWidgets.QWidget):
    signal_finished = QtCore.pyqtSignal()

    def __init__(self, work_environment: ViewerWorkEnv):
        QtWidgets.QWidget.__init__(self)
        self.setWindowTitle('Add to project')

        self.ui = Ui_Form()
        self.ui.setupUi(self)

        self.setWindowTitle('Add to Project')

        self.work_environment = work_environment

        self.ui.radioButtonAddToDataFrame.clicked.connect(self._disable_sample_id_text_entry)
        self.ui.radioButtonAddToDataFrame.clicked.connect(lambda x: self.ui.checkBoxSaveChanges.setChecked(False))

        if self.work_environment.sample_id != '' and self.work_environment.sample_id is not None:
            self.ui.lineEditAnimalID.setText(self.work_environment.sample_id.split('-_-')[0])
            self.ui.lineEditTrialID.setText(self.work_environment.sample_id.split('-_-')[1])
            self.ui.radioButtonSaveChanges.setVisible(True)
            self.ui.checkBoxSaveChanges.clicked.connect(self._disable_sample_id_text_entry)

            self.ui.textBoxComments.setPlainText(self.work_environment.comments)
        else:
            self.ui.radioButtonSaveChanges.setVisible(False)
            self.ui.radioButtonSaveChanges.setChecked(False)
            self.ui.checkBoxSaveChanges.setChecked(False)
            self.ui.checkBoxOverwriteImage.setChecked(False)

        self.custom_column_entries = []

        for custom_column in configuration.proj_cfg.options('CUSTOM_COLUMNS'):
            data_type = configuration.proj_cfg['CUSTOM_COLUMNS'][custom_column]

            val = None

            if self.work_environment.sample_id != '' and self.work_environment.sample_id is not None:
                try:
                    sample_id_df = get_project_manager().get_sample_id_rows(self.work_environment.sample_id)
                    if not sample_id_df.empty:
                        val = sample_id_df[custom_column].iloc[0]
                except:
                    pass

            if data_type == 'bool':
                combo_box = QtWidgets.QComboBox(self)
                combo_box.setObjectName(custom_column)

                combo_box.addItems(['True', 'False'])

                label = QtWidgets.QLabel(self)
                label.setText(custom_column)

                self.ui.verticalLayout.insertWidget(2, combo_box)
                self.ui.verticalLayout.insertWidget(2, label)

                self.custom_column_entries.append(combo_box)

                if val is not None:
                    try:
                        if val is True:
                            combo_box.setCurrentIndex(0)
                        elif val is False:
                            combo_box.setCurrentIndex(1)
                    except:
                        pass

            else:
                line_edit = QtWidgets.QLineEdit(self)
                line_edit.setObjectName(custom_column)
                line_edit.setPlaceholderText(custom_column + ' :' + data_type)

                self.ui.verticalLayout.insertWidget(2, line_edit)

                if data_type == 'int64':
                    line_edit.setValidator(QtGui.QIntValidator())
                elif data_type == 'float64':
                    line_edit.setValidator(QtGui.QDoubleValidator())

                self.custom_column_entries.append(line_edit)

                if val is not None:
                    try:
                        line_edit.setText(str(val))
                    except:
                        pass

        self.ui.checkBoxSaveChanges.setVisible(False)

        self.ui.btnProceed.clicked.connect(self.slot_proceed)

    def _disable_sample_id_text_entry(self):
        if self.ui.radioButtonSaveChanges.isChecked() and self.ui.checkBoxSaveChanges.isChecked():
            animal_id = self.work_environment.sample_id.split('-_-')[0]
            trial_id = self.work_environment.sample_id.split('-_-')[1]

            self.ui.lineEditAnimalID.setText(animal_id)
            self.ui.lineEditAnimalID.setDisabled(True)

            self.ui.lineEditTrialID.setText(trial_id)
            self.ui.lineEditTrialID.setDisabled(True)

        else:
            self.ui.lineEditAnimalID.setEnabled(True)
            self.ui.lineEditTrialID.setEnabled(True)

    def update_work_environment_dicts(self):
        animal_id = self.ui.lineEditAnimalID.text()
        trial_id = self.ui.lineEditTrialID.text()
        sample_id = animal_id + '-_-' + trial_id

        self.work_environment.sample_id = sample_id

        self.work_environment.comments = self.ui.textBoxComments.toPlainText()
        self.work_environment.custom_cols = {}
        for entry in self.custom_column_entries:
            if isinstance(entry, QtWidgets.QLineEdit):
                val = entry.text()

                if configuration.proj_cfg['CUSTOM_COLUMNS'][entry.objectName()] == 'int64':
                    val = int64(val)
                elif configuration.proj_cfg['CUSTOM_COLUMNS'][entry.objectName()] == 'float64':
                    val = float64(val)

            elif isinstance(entry, QtWidgets.QComboBox):
                if entry.currentText() == 'True':
                    val = True
                elif entry.currentText() == 'False':
                    val = False

            self.work_environment.custom_cols.update({entry.objectName(): val})

    def must_enter_sample_id_dialog(self):
        QtWidgets.QMessageBox.warning(self, 'Animal & Trial ID', 'You must enter Animal ID and Trial ID')

    def slot_proceed(self):
        if self.ui.lineEditAnimalID == '':
            self.must_enter_sample_id_dialog()
            return
        if self.ui.lineEditTrialID == '':
            self.must_enter_sample_id_dialog()
            return

        animal_id = self.ui.lineEditAnimalID.text()
        trial_id = self.ui.lineEditTrialID.text()
        sample_id = animal_id + '-_-' + trial_id

        if (sample_id in get_project_manager().dataframe['SampleID'].values) and not self.check_save_changes():
            QtWidgets.QMessageBox.warning(self, 'SampleID exists in project',
                                          'The combination of animal ID and '
                                          'trial ID already exists in the project dataframe. '
                                          'You must choose a unique combination.')
            return

        self.setDisabled(True)
        self.label_wait = QtWidgets.QLabel(self)
        self.label_wait.setText('Please wait...')
        self.ui.verticalLayout.addWidget(self.label_wait)

        self.update_work_environment_dicts()

        if self.ui.radioButtonAddToDataFrame.isChecked():
            self.add_to_dataframe()

        elif self.check_save_changes():
            self.save_changes_to_sample()

    def check_save_changes(self):
        if self.ui.radioButtonSaveChanges.isChecked() and self.ui.checkBoxSaveChanges.isChecked():
            return True
        else:
            return False

    def add_to_dataframe(self):
        dicts_to_append = self.work_environment.to_pandas(configuration.proj_path)
        get_project_manager().append_to_dataframe(dicts_to_append)
        self.work_environment.saved = True
        self.label_wait.setText('FINISHED!')
        # self.signal_finished.emit()

    def save_changes_to_sample(self):
        try:
            save_tiff = self.ui.checkBoxOverwriteImage.isChecked()
            dicts_to_append = self.work_environment.to_pandas(configuration.proj_path, modify_options={'overwrite_img_seq': save_tiff})
        except:
            QtWidgets.QMessageBox.warning(self, 'Exception while trying to overwrite',
                                          'The following exception was raised while trying to package the current'
                                          'work environment to overwrite the SampleID rows: ' + traceback.format_exc())
            return

        get_project_manager().change_sample_rows(self.work_environment.sample_id, dicts_to_append)
        self.work_environment.saved = True
        self.label_wait.setText('FINISHED!')
        # self.signal_finished.emit()

    def close(self):
        self.signal_finished.emit()
        super(AddToProjectDialog, self).close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 25

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""
import pandas as pd
import numpy as np
from ...common import configuration


def empty_df(cols=None, addCols=[]):
    """
    Just returns an empty DataFrame based on columns in the project's config.cfg file. Only really used when a new
    project is started.

    :param cols: column names
    :type cols:  list
    :return:     empty pandas DataFrame with names from list passed in
    :rtype:      pd.DataFrame
    """
    if cols is None:
        include = configuration.proj_cfg.options('INCLUDE')
        exclude = configuration.proj_cfg.options('EXCLUDE')

        cols = include + exclude

    cols = cols + addCols

    if type(cols) is list:
        cols = set(cols)

    return pd.DataFrame(data=None, columns=cols)


def fix_fp_errors(n):
    fix = np.round(n, decimals=1) + 0.0
    return fix



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 14 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""
from typing import List, Any, Union

from PyQt5.QtGui import QColor

from .window_pytemplate import *
import pandas as pd
import pickle
import scipy
from ....pyqtgraphCore import GraphicsLayoutWidget, mkColor
from ....pyqtgraphCore.console import ConsoleWidget
from matplotlib import cm as matplotlib_color_map
import numpy as np
from ....analysis import Transmission, organize_dataframe_columns
from ....common import configuration
import os


class PlotWindow(QtWidgets.QMainWindow):
    # __metaclass__ = abc.ABCMeta
    signal_params_updated = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        super(PlotWindow, self).__init__(parent)

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.transmissions = None
        self.merged_transmission = None
        self.dataframe = pd.DataFrame()
        self.data_columns = []
        self.data_columns_dtypes = []

        self.groups = []
        self.group_dataframes = []
        self.uuid_column = ''

        self.graphicsViews = {}

        self.ui.listWidgetDataColumns.setSortingEnabled(True)

        self.ui.btnApplyAll.clicked.connect(self.update_params)

        ns = {'np': np,
              'pickle': pickle,
              'scipy': scipy,
              'pd': pd,
              'graphicsViews': self.graphicsViews,
              'this': self,
              }

        txt = "Namespaces:\n" \
              "pickle as pickle\n" \
              "numpy as 'np'\n" \
              "scipy as 'scipy'\n" \
              "pd as pandas\n" \
              "graphicsViews (plot tabs) as graphicsViews\n" \
              "self as 'this'\n\n" \

        cmd_history_file = os.path.join(configuration.console_history_path, 'plot_window.pik')

        self.ui.dockConsole.setWidget(ConsoleWidget(namespace=ns, text=txt, historyFile=cmd_history_file))

        self.ui.dockConsole.hide()

        self.ui.comboBoxShape.hide()
        self.ui.checkBoxUseDifferentShapes.hide()

        self.old_column_selection = {'group': None,
                                     'shape': None,
                                     'uuid': None
                                     }

    @property
    def status_bar(self) -> QtWidgets.QStatusBar:
        return self.statusBar()

    def set_old_selections(self):
        if self.old_column_selection['group'] is not None:
            ix = self.ui.comboBoxGrouping.findText(self.old_column_selection['group'])
            if ix != -1:
                self.ui.comboBoxGrouping.setCurrentIndex(ix)

        if self.old_column_selection['shape'] is not None:
            ix = self.ui.comboBoxShape.findText(self.old_column_selection['shape'])
            if ix != -1:
                self.ui.comboBoxShape.setCurrentIndex(ix)

        if self.old_column_selection['uuid'] is not None:
            ix = self.ui.comboBoxUUIDColumn.findText(self.old_column_selection['uuid'])
            if ix != -1:
                self.ui.comboBoxUUIDColumn.setCurrentIndex(ix)
        # if self.old_column_selection['data'] is not None:
        #     item = self.ui.listWidgetDataColumns.get(self.old_column_selection['data'], QtCore.Qt.MatchExactly).text()
        #     self.ui.listWidgetDataColumns.setSelection()

    def update_input_transmissions(self, transmissions: list):
        self.transmissions = transmissions

        dfs = [t.df for t in self.transmissions]

        self.dataframe = pd.concat(dfs)#, axis=1)
        self.dataframe.reset_index(drop=True, inplace=True)
        try:
            self.dataframe.drop(columns=['index'], inplace=True)
        except:
            pass
        #self.dataframe = self.dataframe.sample(n=100, axis=0)

        columns = self.dataframe.columns.tolist()

        columns.sort()

        self.merged_transmission = Transmission.merge(self.transmissions)

        dcols, ccols, ucols = organize_dataframe_columns(self.merged_transmission.df.columns)

        self.ui.listWidgetDataColumns.clear()
        self.ui.listWidgetDataColumns.addItems(dcols)

        self.ui.comboBoxGrouping.clear()
        self.ui.comboBoxGrouping.addItems(ccols)

        self.ui.comboBoxShape.clear()
        self.ui.comboBoxShape.addItems(ccols)

        self.ui.comboBoxDataPointTracerCurveColumn.clear()
        self.ui.comboBoxDataPointTracerCurveColumn.addItems(dcols)

        self.ui.comboBoxUUIDColumn.clear()
        self.ui.comboBoxUUIDColumn.addItems(ucols)
        # self.ui.comboBoxUUIDColumn.setCurrentIndex(columns.index(next(column for column in columns if 'uuid' in column)))

        self.set_old_selections()

    def add_plot_tab(self, title: str):
        self.graphicsViews.update({title: GraphicsLayoutWidget(parent=self.ui.tabWidget)})
        self.ui.tabWidget.addTab(self.graphicsViews[title], title)
        i = self.ui.tabWidget.count() - 1
        self.ui.tabWidget.widget(i).setLayout(QtWidgets.QVBoxLayout())

    def update_params(self):
        self.data_columns.clear()
        self.data_columns = [item.text() for item in self.ui.listWidgetDataColumns.selectedItems()]

        for col in self.data_columns:
            dtype = type(self.dataframe[col].iloc[0])
            if dtype is np.NaN:
                i = 0
                while dtype is np.NaN:
                    dtype = type(self.dataframe[col].iloc[i])
                    i += 1

            self.data_columns_dtypes.append(dtype)

        if self.ui.radioButtonGroupBySingleColumn.isChecked():
            self.grouping_column = self.ui.comboBoxGrouping.currentText()
            self.groups = list(set(self.dataframe[self.grouping_column].tolist()))
        else:
            return
        self.group_dataframes = [self.dataframe[self.dataframe[self.grouping_column] == group] for group in self.groups]
        self.shape_column = self.ui.comboBoxShape.currentText()
        self.uuid_column = self.ui.comboBoxUUIDColumn.currentText()

        self.datapoint_tracer_curve_column = self.ui.comboBoxDataPointTracerCurveColumn.currentText()

        self.old_column_selection['group'] = self.grouping_column
        self.old_column_selection['shape'] = self.shape_column
        self.old_column_selection['uuid'] = self.uuid_column

    def auto_colormap(self, number_of_colors: int, color_map: str = 'hsv') -> List[Union[QColor, Any]]:
        cm = matplotlib_color_map.get_cmap(color_map)
        cm._init()
        lut = (cm._lut * 255).view(np.ndarray)
        cm_ixs = np.linspace(0, 210, number_of_colors, dtype=int)

        colors = []
        for ix in range(number_of_colors):
            c = lut[cm_ixs[ix]]
            colors.append(mkColor(c))
        return colors

    def data_types_check(self, accepted_dtypes: list) -> bool:
        errors = []
        for data_column, data_dtype in zip(self.data_columns, self.data_columns_dtypes):
            if data_dtype not in accepted_dtypes:
                errors.append('Unsupported data type <' + str(data_dtype) + '> in data column: ' + data_column)

        if len(errors) > 0:
            QtWidgets.QMessageBox.warning(self, 'Datatype error',
                                          'You cannot use the following data columns:\n' + '\n'.join(errors))
            return False
        else:
            return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 15 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ...pyqtgraphCore import GraphicsLayoutWidget, LegendItem
from ...pyqtgraphCore.Qt import QtCore, QtGui, QtWidgets
import numpy as np
import pandas as pd
from uuid import UUID
from typing import List, SupportsFloat


class PgCurvePlot(QtCore.QObject):
    signal_spot_clicked = QtCore.pyqtSignal(UUID)

    def __init__(self, graphics_view: GraphicsLayoutWidget, parent=None):
        QtCore.QObject.__init__(self, parent)

        self.graphics_view = graphics_view
        self.plots = []
        self.curve_plots = []
        self.title = ''
        self.current_datapoint = None
        self.lastClicked = []
        self.plot_items = []

    def add_plot(self, title: str):
        # plot = PlotDataItem()
        plot = self.graphics_view.addPlot(title=title)
        self.plots.append(plot)

    def add_data_to_plot(self, ix: int, data_series: pd.Series, name, color):
        if ix > len(self.plots) - 1:
            raise IndexError('Plot index out of range.')

        not_nan = data_series.notna()
        yvals = data_series[not_nan].values

        for yval in yvals:
            self.plots[ix].plot(y=yval, pen=color)

    def clear_plot(self):
        for plot in self.plots:
            self.graphics_view.removeItem(plot)
            plot.deleteLater()
        del self.plots
        self.plots = []

        # self.graphics_view.scene().clear()
        # for item in self.graphics_view.items():
        #     self.graphics_view.removeItem(item)

    def export_all_plots(self):
        pass

    def export_plot(self, column: str, filetype: str = 'svg', title: str = None, error_bars: str = 'mean',
                    spots_color=None, spots_outline='black', background_color='black', axis_color='white'):
        pass

    def get_range(self, plot_ix: int) -> list:
        return self.plots[plot_ix].vb.viewRange()

    def set_range(self, plot_ix, x_range: list, y_range: list):
        self.plots[plot_ix].setRange(xRange=x_range, yRange=y_range)

    def create_legend(self, plot_ix):
        legend = LegendItem()
        legend.setParentItem(self.plots[plot_ix])



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ControlWidget(object):
    def setupUi(self, ControlWidget):
        ControlWidget.setObjectName("ControlWidget")
        ControlWidget.resize(320, 828)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(ControlWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_2 = QtWidgets.QLabel(ControlWidget)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.comboBoxDataColumn = ComboBox(ControlWidget)
        self.comboBoxDataColumn.setObjectName("comboBoxDataColumn")
        self.verticalLayout_3.addWidget(self.comboBoxDataColumn)
        self.label_3 = QtWidgets.QLabel(ControlWidget)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.comboBoxLabelsColumn = ComboBox(ControlWidget)
        self.comboBoxLabelsColumn.setObjectName("comboBoxLabelsColumn")
        self.verticalLayout_3.addWidget(self.comboBoxLabelsColumn)
        self.label_5 = QtWidgets.QLabel(ControlWidget)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_3.addWidget(self.label_5)
        self.comboBoxDPTCurveColumn = ComboBox(ControlWidget)
        self.comboBoxDPTCurveColumn.setObjectName("comboBoxDPTCurveColumn")
        self.verticalLayout_3.addWidget(self.comboBoxDPTCurveColumn)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_6 = QtWidgets.QLabel(ControlWidget)
        self.label_6.setAlignment(QtCore.Qt.AlignCenter)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        self.doubleSpinBox_vmin = QtWidgets.QDoubleSpinBox(ControlWidget)
        self.doubleSpinBox_vmin.setDecimals(6)
        self.doubleSpinBox_vmin.setObjectName("doubleSpinBox_vmin")
        self.verticalLayout.addWidget(self.doubleSpinBox_vmin)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_7 = QtWidgets.QLabel(ControlWidget)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_2.addWidget(self.label_7)
        self.doubleSpinBox_vmax = QtWidgets.QDoubleSpinBox(ControlWidget)
        self.doubleSpinBox_vmax.setDecimals(6)
        self.doubleSpinBox_vmax.setObjectName("doubleSpinBox_vmax")
        self.verticalLayout_2.addWidget(self.doubleSpinBox_vmax)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.pushButton_set_vmin_vmax = QtWidgets.QPushButton(ControlWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_set_vmin_vmax.sizePolicy().hasHeightForWidth())
        self.pushButton_set_vmin_vmax.setSizePolicy(sizePolicy)
        self.pushButton_set_vmin_vmax.setObjectName("pushButton_set_vmin_vmax")
        self.horizontalLayout_2.addWidget(self.pushButton_set_vmin_vmax)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.label = QtWidgets.QLabel(ControlWidget)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.listWidgetColorMapsData = ColormapListWidget(ControlWidget)
        self.listWidgetColorMapsData.setObjectName("listWidgetColorMapsData")
        self.verticalLayout_3.addWidget(self.listWidgetColorMapsData)
        self.label_4 = QtWidgets.QLabel(ControlWidget)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.listWidgetColorMapsLabels = ColormapListWidget(ControlWidget)
        self.listWidgetColorMapsLabels.setObjectName("listWidgetColorMapsLabels")
        self.verticalLayout_3.addWidget(self.listWidgetColorMapsLabels)
        self.checkBoxLiveUpdate = QtWidgets.QCheckBox(ControlWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.checkBoxLiveUpdate.setFont(font)
        self.checkBoxLiveUpdate.setObjectName("checkBoxLiveUpdate")
        self.verticalLayout_3.addWidget(self.checkBoxLiveUpdate)
        self.pushButtonPlot = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonPlot.setMinimumSize(QtCore.QSize(0, 46))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonPlot.setFont(font)
        self.pushButtonPlot.setCheckable(False)
        self.pushButtonPlot.setObjectName("pushButtonPlot")
        self.verticalLayout_3.addWidget(self.pushButtonPlot)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButtonSave = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonSave.setMinimumSize(QtCore.QSize(0, 46))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonSave.setFont(font)
        self.pushButtonSave.setCheckable(False)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.horizontalLayout.addWidget(self.pushButtonSave)
        self.pushButtonLoad = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonLoad.setMinimumSize(QtCore.QSize(0, 46))
        font = QtGui.QFont()
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonLoad.setFont(font)
        self.pushButtonLoad.setCheckable(False)
        self.pushButtonLoad.setObjectName("pushButtonLoad")
        self.horizontalLayout.addWidget(self.pushButtonLoad)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.checkBoxShowStimuli = QtWidgets.QCheckBox(ControlWidget)
        self.checkBoxShowStimuli.setObjectName("checkBoxShowStimuli")
        self.verticalLayout_3.addWidget(self.checkBoxShowStimuli)
        self.pushButtonShowHideConsole = QtWidgets.QPushButton(ControlWidget)
        self.pushButtonShowHideConsole.setMinimumSize(QtCore.QSize(0, 38))
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonShowHideConsole.setFont(font)
        self.pushButtonShowHideConsole.setCheckable(True)
        self.pushButtonShowHideConsole.setObjectName("pushButtonShowHideConsole")
        self.verticalLayout_3.addWidget(self.pushButtonShowHideConsole)

        self.retranslateUi(ControlWidget)
        QtCore.QMetaObject.connectSlotsByName(ControlWidget)
        ControlWidget.setTabOrder(self.comboBoxDataColumn, self.comboBoxLabelsColumn)
        ControlWidget.setTabOrder(self.comboBoxLabelsColumn, self.comboBoxDPTCurveColumn)
        ControlWidget.setTabOrder(self.comboBoxDPTCurveColumn, self.doubleSpinBox_vmin)
        ControlWidget.setTabOrder(self.doubleSpinBox_vmin, self.doubleSpinBox_vmax)
        ControlWidget.setTabOrder(self.doubleSpinBox_vmax, self.pushButton_set_vmin_vmax)
        ControlWidget.setTabOrder(self.pushButton_set_vmin_vmax, self.listWidgetColorMapsData)
        ControlWidget.setTabOrder(self.listWidgetColorMapsData, self.listWidgetColorMapsLabels)
        ControlWidget.setTabOrder(self.listWidgetColorMapsLabels, self.checkBoxLiveUpdate)
        ControlWidget.setTabOrder(self.checkBoxLiveUpdate, self.pushButtonPlot)
        ControlWidget.setTabOrder(self.pushButtonPlot, self.pushButtonSave)
        ControlWidget.setTabOrder(self.pushButtonSave, self.pushButtonLoad)
        ControlWidget.setTabOrder(self.pushButtonLoad, self.checkBoxShowStimuli)
        ControlWidget.setTabOrder(self.checkBoxShowStimuli, self.pushButtonShowHideConsole)

    def retranslateUi(self, ControlWidget):
        _translate = QtCore.QCoreApplication.translate
        ControlWidget.setWindowTitle(_translate("ControlWidget", "Form"))
        self.label_2.setText(_translate("ControlWidget", "Data columns"))
        self.label_3.setText(_translate("ControlWidget", "Labels column"))
        self.label_5.setText(_translate("ControlWidget", "DPT curve column"))
        self.label_6.setText(_translate("ControlWidget", "vmin"))
        self.label_7.setText(_translate("ControlWidget", "vmax"))
        self.pushButton_set_vmin_vmax.setText(_translate("ControlWidget", "Set\n"
"vmin vmax"))
        self.label.setText(_translate("ControlWidget", "Data colormap:"))
        self.label_4.setText(_translate("ControlWidget", "Legend colormap:"))
        self.checkBoxLiveUpdate.setText(_translate("ControlWidget", "Live update from input transmission"))
        self.pushButtonPlot.setText(_translate("ControlWidget", "Plot"))
        self.pushButtonSave.setText(_translate("ControlWidget", "Save"))
        self.pushButtonLoad.setText(_translate("ControlWidget", "Load"))
        self.checkBoxShowStimuli.setText(_translate("ControlWidget", "Show Stimuli"))
        self.pushButtonShowHideConsole.setText(_translate("ControlWidget", "Show/Hide Console"))

from ....pyqtgraphCore.widgets.ComboBox import ComboBox
from ...utils import ColormapListWidget



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 15 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""


from PyQt5 import QtCore, QtWidgets


class ListWidgetDialog(QtWidgets.QWidget):
    def __init__(self):
        QtWidgets.QWidget.__init__(self)
        self.vertical_layout = QtWidgets.QVBoxLayout(self)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)

        self.label = QtWidgets.QLabel()
        self.vertical_layout.addWidget(self.label)

        self.listWidget = QtWidgets.QListWidget()
        self.vertical_layout.addWidget(self.listWidget)

        self.btnCancel = QtWidgets.QPushButton()
        self.btnCancel.setText('Cancel')
        self.vertical_layout.addWidget(self.btnCancel)

        self.btnOK = QtWidgets.QPushButton()
        self.btnOK.setText('OK')
        self.vertical_layout.addWidget(self.btnOK)
        self.show()
        self.resize(400, 300)


if __name__ == '__main__':
    app = QtWidgets.QApplication([])
    lwd = ListWidgetDialog()
    lw = QtWidgets.QListWidget()
    lwd.insert_list_widget(lw)


    app.exec_()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from PyQt5 import QtCore, QtWidgets
from ...pyqtgraphCore.widgets.MatplotlibWidget import MatplotlibWidget
import numpy as np
# from seaborn import clustermap as sns_clustermap
from seaborn.matrix import ClusterGrid, _matrix_mask
from matplotlib.patches import Rectangle as RectangularPatch
from matplotlib.lines import Line2D
from matplotlib.transforms import Bbox
from matplotlib.patches import Patch as MPatch
from matplotlib import rcParams
from matplotlib import pyplot as plt
from matplotlib.widgets import RectangleSelector
from pandas import Series
from ..utils import get_colormap, map_labels_to_colors
from typing import *
import pandas as pd
from matplotlib import gridspec
from functools import partial


class CustomClusterGrid(ClusterGrid):
    """Slightly modified from Seaborn ClusterGrid so that an existing figure instance can be used"""
    def __init__(self, data, fig, pivot_kws=None, z_score=None, standard_scale=None,
                 figsize=None, row_colors=None, col_colors=None, mask=None):
        """Grid object for organizing clustered heatmap input on to axes"""

        if isinstance(data, pd.DataFrame):
            self.data = data
        else:
            self.data = pd.DataFrame(data)

        self.data2d = self.format_data(self.data, pivot_kws, z_score,
                                       standard_scale)

        self.mask = _matrix_mask(self.data2d, mask)

        if figsize is None:
            width, height = 10, 10
            figsize = (width, height)
        if fig is None:
            self.fig = plt.figure(figsize=figsize)
        else:
            self.fig = fig

        self.row_colors, self.row_color_labels = \
            self._preprocess_colors(data, row_colors, axis=0)
        self.col_colors, self.col_color_labels = \
            self._preprocess_colors(data, col_colors, axis=1)

        width_ratios = self.dim_ratios(self.row_colors,
                                       figsize=figsize,
                                       axis=1)

        height_ratios = self.dim_ratios(self.col_colors,
                                        figsize=figsize,
                                        axis=0)
        nrows = 3 if self.col_colors is None else 4
        ncols = 3 if self.row_colors is None else 4

        self.gs = gridspec.GridSpec(nrows, ncols, wspace=0.01, hspace=0.01,
                                    width_ratios=width_ratios,
                                    height_ratios=height_ratios)

        self.ax_row_dendrogram = self.fig.add_subplot(self.gs[nrows - 1, 0:2])
        self.ax_col_dendrogram = self.fig.add_subplot(self.gs[0:2, ncols - 1])
        self.ax_row_dendrogram.set_axis_off()
        self.ax_col_dendrogram.set_axis_off()

        self.ax_row_colors = None
        self.ax_col_colors = None

        if self.row_colors is not None:
            self.ax_row_colors = self.fig.add_subplot(
                self.gs[nrows - 1, ncols - 2])
        if self.col_colors is not None:
            self.ax_col_colors = self.fig.add_subplot(
                self.gs[nrows - 2, ncols - 1])

        self.ax_heatmap = self.fig.add_subplot(self.gs[nrows - 1, ncols - 1])

        # colorbar for scale to left corner
        self.cax = self.fig.add_subplot(self.gs[0, 0])

        self.dendrogram_row = None
        self.dendrogram_col = None


class Heatmap(MatplotlibWidget):
    """Heatmap plot variant"""
    sig_selection_changed = QtCore.pyqtSignal(tuple)  #: Emits indices of data coordinates (x, y) from mouse-click events on the heatmap

    def __init__(self, highlight_mode='row'):
        """

        :param highlight_mode: The selection mode for the heatmap. One of either 'row' or 'item'
        """
        MatplotlibWidget.__init__(self)
        rcParams['image.interpolation'] = None

        self.data = None  #: 2D numpy array of the heatmap data

        self.selector = Selection()  #: Selection instance that organizes mouse click events on the heatmap
        self.selector.sig_selection_changed.connect(self.sig_selection_changed.emit)
        self.selector.sig_selection_changed.connect(self.scroll_selector)

        self.stimulus_indicators = []
        self.highlight_mode = highlight_mode  #: selection mode, either 'row' or 'item

        self.plot = None  #: ClusterGrid object instance containing the plot Axes

        self.cid_heatmap = None
        self.cid_ax_row_colors = None
        self.cid_ax_row_dendrogram = None

        self.xcid_heatmap = None
        self.xcid_ax_row_colors = None
        self.xcid_ax_row_dendrogram = None

        self._previous_ylims = None
        self.max_ylim = None
        self.min_ylim = None
        self.xlims = None

    def set(self, data: np.ndarray, *args, ylabels: Union[Series, np.ndarray, list] = None, ylabels_cmap: str = 'tab20',
            cluster_kwargs: dict = None, **kwargs):
        """
        :param data:    2D numpy array
        :param args:    Additional args that are passed to sns.heatmap()
        :param ylabels: Labels used to create the ylabels bar
        :param ylabels_cmap: colormap for the ylabels bar
        :param cluster_kwargs: keywoard arguments for visualizing hierarchical clustering
        :param kwargs:  Additional kwargs that are passed to sns.heatmap()
        """
        self.data = data

        if isinstance(ylabels, Series):
            ylabels = ylabels.values
        if ylabels is not None:
            mapper = get_colormap(ylabels, ylabels_cmap)
            row_colors = list(map(mapper.get, ylabels))
        else:
            row_colors = None

        if 'xlabels' in kwargs.keys():
            xlabels = kwargs.pop('xlabels')

            if isinstance(xlabels, Series):
                xlabels = xlabels.values

            xlabels_mapper = get_colormap(xlabels, ylabels_cmap)
            col_colors = list(map(xlabels_mapper.get, xlabels))
        else:
            col_colors = None

        cluster_kwarg_keys = ['row_cluster', 'row_linkage', 'col_cluster', 'col_linkage', 'colorbar_kws', 'metric', 'method']

        if cluster_kwargs is None:
            cluster_kwargs = dict.fromkeys(cluster_kwarg_keys)
            self.cluster_color_mapper = None
        else:
            if 'cluster_labels' in cluster_kwargs.keys():
                cluster_labels = cluster_kwargs.pop('cluster_labels')

                self.cluster_color_mapper = get_colormap(cluster_labels, cmap='tab10')
                clus_colors = list(map(self.cluster_color_mapper.get, cluster_labels))

                row_colors = [clus_colors, row_colors]
            else:
                self.cluster_color_mapper = None

            for key in cluster_kwarg_keys:
                if key not in cluster_kwargs.keys():
                    cluster_kwargs[key] = None

        if self.fig is not None:
            self.fig.clear()

        self.plot = CustomClusterGrid(data=data, fig=self.fig, figsize=None, row_colors=row_colors, col_colors=col_colors,
                                      z_score=None, standard_scale=None, mask=None)
        self.plot.plot(*args, **cluster_kwargs, **kwargs)

        if ylabels is not None:
            self.create_ylabels_legend(mapper, self.cluster_color_mapper)

        self._previous_ylims = self.plot.ax_heatmap.get_ylim()

        self.xlims = {self.plot.ax_heatmap: self.plot.ax_heatmap.get_xlim(),
                      self.plot.ax_row_colors: self.plot.ax_row_colors.get_xlim(),
                      self.plot.ax_row_dendrogram: self.plot.ax_row_dendrogram.get_xlim()
                      }

        self.max_ylim = self.plot.ax_heatmap.get_ylim()[0]
        self.min_ylim = self.plot.ax_heatmap.get_ylim()[1]

        self.draw()
        self.toolbar.update()

        self.connect_ylim_callbacks()
        self.connect_xlim_callbacks()

        self.selector.set(self, mode=self.highlight_mode)
        # if isinstance(self.selector, Selection):
        #     self.selector.sig_selection_changed.disconnect(self.sig_selection_changed.emit)
        #     del self.selector

        # self.selector = Selection(self, mode=self.highlight_mode)

    def block_callbacks(func):
        """Block callbacks, used when the plot x and y limits change due to user interaction"""
        def fn(self, *args, **kwargs):
            self.disconnect_xlim_callbacks()
            self.disconnect_ylim_callbacks()
            try:
                ret = func(self, *args, **kwargs)
            finally:
                self.connect_xlim_callbacks()
                self.connect_ylim_callbacks()
            return ret
        return fn

    def connect_xlim_callbacks(self):
        self.xcid_heatmap = self.plot.ax_heatmap.callbacks.connect('xlim_changed', self.on_xlim_changed)
        self.xcid_ax_row_colors = self.plot.ax_row_colors.callbacks.connect('xlim_changed', self.on_xlim_changed)
        self.xcid_ax_row_dendrogram = self.plot.ax_row_dendrogram.callbacks.connect('xlim_changed', self.on_xlim_changed)

    def disconnect_xlim_callbacks(self):
        self.plot.ax_heatmap.callbacks.disconnect(self.xcid_heatmap)
        self.plot.ax_row_colors.callbacks.disconnect(self.xcid_ax_row_colors)
        self.plot.ax_row_dendrogram.callbacks.disconnect(self.xcid_ax_row_dendrogram)

    def connect_ylim_callbacks(self):
        self.cid_heatmap = self.plot.ax_heatmap.callbacks.connect('ylim_changed', self.on_ylim_changed)
        self.cid_ax_row_colors = self.plot.ax_row_colors.callbacks.connect('ylim_changed', self.on_ylim_changed)
        self.cid_ax_row_dendrogram = self.plot.ax_row_dendrogram.callbacks.connect('ylim_changed', self.on_ylim_changed)

    def disconnect_ylim_callbacks(self):
        self.plot.ax_heatmap.callbacks.disconnect(self.cid_heatmap)
        self.plot.ax_row_colors.callbacks.disconnect(self.cid_ax_row_colors)
        self.plot.ax_row_dendrogram.callbacks.disconnect(self.cid_ax_row_dendrogram)

    @block_callbacks
    def on_xlim_changed(self, ax):
        """Reset the x limits"""
        ax.set_xlim(self.xlims[ax])

    @block_callbacks
    def on_ylim_changed(self, ax):
        if ax is self.plot.ax_row_dendrogram:
            # divide by 10
            lims = tuple(map(lambda x: x / 10, ax.get_ylim()))
        else:
            lims = ax.get_ylim()

        if lims[0] > self.max_ylim or lims[1] < self.min_ylim:
            self.y_lims_exceeded()
            return

        self.set_y_lims(lims, skip_axes=ax)


        self._previous_ylims = lims

    @block_callbacks
    def y_lims_exceeded(self):
        lims = self._previous_ylims
        self.plot.ax_heatmap.set_ylim(lims)
        self.plot.ax_row_colors.set_ylim(lims)
        self.plot.ax_row_dendrogram.set_ylim(tuple(map(lambda x: x*10, lims)))

    def create_ylabels_legend(self, ylabels_mapper, cluster_mapper):
        self.plot.ax_col_dendrogram.cla()
        handles = [MPatch(color=ylabels_mapper[k], label=k) for k in ylabels_mapper.keys()]

        if cluster_mapper is not None:
            handles = [MPatch(color=cluster_mapper[k], label=k) for k in cluster_mapper.keys()] + handles

        self.plot.ax_col_dendrogram.legend(handles=handles, ncol=int(np.sqrt(len(handles))))
        self.plot.ax_col_dendrogram.get_xaxis().set_visible(False)
        self.plot.ax_col_dendrogram.get_yaxis().set_visible(False)

    def add_stimulus_indicator(self, start: int, end: int, color: str):
        """
        Add lines to indicate the start and end of a stimulus or behavioral period

        :param start: start index
        :param end: end index
        :param color: line color
        """
        for t in [start, end]:
            x = np.array([t, t])
            y = np.array([0, self.data.shape[0]])
            line = Line2D(x, y, lw=3, color=color)
            self.stimulus_indicators.append(line)
            self.plot.add_line(line)

    def set_y_lims(self, lims: Tuple[int, int], skip_axes = None):
        if skip_axes is not self.plot.ax_heatmap:
            self.plot.ax_heatmap.set_ylim(lims)

        if skip_axes is not self.plot.ax_row_colors:
            self.plot.ax_row_colors.set_ylim(lims)

        if skip_axes is not self.plot.ax_row_dendrogram:
            # scale up by 10 for this ax
            lims_ = tuple(map(lambda x: x * 10, lims))
            self.plot.ax_row_dendrogram.set_ylim(lims_)

    def get_y_lims(self) -> Tuple[int, int]:
        if not self.plot.ax_heatmap.get_ylim() == self.plot.ax_row_colors.get_ylim():
            raise ValueError('Axes y_lims not in sync, something wrong')

        return self.plot.ax_heatmap.get_ylim()

    @QtCore.pyqtSlot(tuple)
    def scroll_selector(self, ixs: Tuple[int, int]):
        upper, lower = self.get_y_lims()
        if self.highlight_mode == 'row':
            if ixs[1] + 2 > upper:
                if ixs[1] + 2 > self.max_ylim:
                    return

                new_upper = min(ixs[1] + 10, self.max_ylim)
                new_lower = max(self.min_ylim, lower + 10)

            elif ixs[1] - 2 < lower:
                if ixs[1] - 2 < self.min_ylim:
                    return

                new_lower = max(self.min_ylim, ixs[1] - 10)
                new_upper = min(upper - 10, self.max_ylim)

            else:
                return

            lims = (new_upper, new_lower)

            self.set_y_lims(lims)


class Selection(QtCore.QObject):
    sig_selection_changed = QtCore.pyqtSignal(tuple)
    sig_multi_selection_changed = QtCore.pyqtSignal(list)

    def __init__(self): #, heatmap_obj, mode: str = 'row'):
        QtCore.QObject.__init__(self)
        self.canvas = None #heatmap_obj.canvas
        self.plot = None #heatmap_obj.plot
        self.heatmap = None #heatmap_obj

        self.RS = None
        self.current_ix = None
        self._highlight = None
        self._start_ixs = None
        self.multi_select_mode = False

        self._multiselect = False
        self.multi_selection_list = list()
        self.mode = None

    def set(self, heatmap_obj, mode: str = 'row'):
        self._highlight = None
        self.current_ix = None

        self.canvas = heatmap_obj.plot.fig.canvas
        self.plot = heatmap_obj.plot.ax_heatmap
        self.heatmap = heatmap_obj

        self.mode = mode

        if mode == 'row':
            self.canvas.mpl_connect('button_press_event', self.select_row)
            self.canvas.setFocusPolicy(QtCore.Qt.ClickFocus)
            self.canvas.setFocus()
            self.canvas.mpl_connect('key_press_event', self.on_key_press_event)

        elif mode == 'item':
            self.canvas.mpl_connect('button_press_event', self.select_item)
            self.canvas.setFocusPolicy(QtCore.Qt.ClickFocus)
            self.canvas.setFocus()
            self.canvas.mpl_connect('key_press_event', self.set_multi_select_mode_on)
            self.canvas.mpl_connect('key_release_event', self.set_multi_select_mode_off)
        else:
            raise ValueError("Invalid selection mode: '" + str(mode) + "' Valid modes are: 'row', 'item' or 'span'.")

    def _clear_highlight(self):
        if self._highlight is not None:
            self._highlight.remove()
        self._highlight = None

    def update_selection(self, ix: tuple):
        if not self.multi_select_mode:
            self._clear_highlight()
            self.multi_selection_list.clear()

        self.current_ix = ix
        self.sig_selection_changed.emit(self.current_ix)

        if self.multi_select_mode:
            self.multi_selection_list.append(ix)
            self.sig_multi_selection_changed.emit(self.multi_selection_list)

    def select_row(self, ev):
        if type(ev) is int:
            y = ev
        else:
            if ev.button == 3:
                self._clear_highlight()
                self.canvas.draw()
                return

            y = ev.ydata

            if y is None:
                return

            y = int(y)

        ix = (0, y)
        self.update_selection(ix)
        self._draw_row_selection(y)

    def _draw_row_selection(self, y):
        self._highlight = self.plot.add_patch(RectangularPatch((0, y), self.heatmap.data.shape[1], 1,
                                                               facecolor='w', edgecolor='k', lw=3, alpha=0.2))
        self.canvas.draw()

    def on_key_press_event(self, ev):
        if ev.key == 'up':
            ixs = max(self.current_ix[1] - 1, self.heatmap.min_ylim)

        elif ev.key == 'down':
            ixs = min(self.current_ix[1] + 1, self.heatmap.max_ylim)

        else:

            return
        # elif ev.key == 'left':
        #     pass
        # elif ev.key == 'right':
        #     pass

        self.select_row(int(ixs))

    def set_multi_select_mode_on(self, ev):
        if ev.key == 'control':
            self.multi_select_mode = True

    def set_multi_select_mode_off(self, ev):
        if ev.key == 'control':
            self.multi_select_mode = False

    def select_item(self, ev):
        if type(ev) is not tuple:
            ix = (int(ev.xdata), int(ev.ydata))

            self.update_selection(ix)
            self._draw_item_selection(ix)

    def _draw_item_selection(self, ix: tuple):
        self._highlight = self.plot.add_patch(RectangularPatch(ix, 1, 1, facecolor='w', edgecolor='k', lw=3, alpha=0.2))
        self.canvas.draw()

    # def toggle_selector(self, event):
    #     if event.key in ['Q', 'q'] and self.RS.active:
    #         print('RectangleSelector deactivated.')
    #         self.RS.set_active(False)
    #     if event.key in ['A', 'a'] and not self.RS.active:
    #         print('RectangleSelector activated.')
    #         self.RS.set_active(True)

    # def line_select_callback(self, eclick, erelease):
    #     'eclick and erelease are the press and release events'
    #     x1, y1 = int(eclick.xdata), int(eclick.ydata)
    #     x2, y2 = int(erelease.xdata), int(erelease.ydata)
    #     self.signal_selection_changed.emit((x1, y1, x2, y2))
    #     print("(%3.2f, %3.2f) --> (%3.2f, %3.2f)" % (x1, y1, x2, y2))
    #     print(" The button you used were: %s %s" % (eclick.button, erelease.button))
            

if __name__ == '__main__':
    import pickle
    from sklearn.metrics import pairwise_distances
    t = pickle.load(open('/home/kushal/Sars_stuff/mesmerize_toy_datasets/cesa_hnk1_raw_data.trn', 'rb'))
    df = t['df']
    
    data = np.vstack(df._RAW_CURVE.apply(lambda x: x[:2998]).values)
    
    dists = pairwise_distances(data)
    
    app = QtWidgets.QApplication([])
    w = Heatmap(highlight_mode='item')
    w.set(dists, cmap='jet', ylabels=df.promoter)
    
    w.show()
    app.exec_()



# -*- coding: utf-8 -*-
import sys
from ...Qt import QtCore, QtGui
from ...widgets.SpinBox import SpinBox
#from ...SignalProxy import SignalProxy
from ...WidgetGroup import WidgetGroup
#from ColorMapper import ColorMapper
from ..Node import Node
import numpy as np
from ...widgets.ColorButton import ColorButton
from ....analysis import Transmission
from tqdm import tqdm
try:
    import metaarray
    HAVE_METAARRAY = True
except:
    HAVE_METAARRAY = False

from ...widgets.ComboBox import ComboBox as QComboBox
from ...widgets.ListWidget import ListWidget
from ...widgets.LineEdit import LineEdit
from ...widgets.KwargPlainTextEdit import KwargPlainTextEdit
from ....analysis import organize_dataframe_columns
# from PyQt5.QtWidgets import QLineEdit as LineEdit
import logging


logger = logging.getLogger()


def generateUi(opts):
    """Convenience function for generating common UI types"""
    widget = QtGui.QWidget(parent=None)
    l = QtGui.QFormLayout()
    l.setSpacing(0)
    widget.setLayout(l)
    ctrls = {}
    row = 0
    for opt in opts:
        if len(opt) == 2:
            k, t = opt
            o = {}
        elif len(opt) == 3:
            k, t, o = opt
        else:
            raise Exception("Widget specification must be (name, type) or (name, type, {opts})")
        if t == 'intSpin':
            w = QtGui.QSpinBox()
            if 'max' in o:
                w.setMaximum(o['max'])
            if 'min' in o:
                w.setMinimum(o['min'])
            if 'value' in o:
                w.setValue(o['value'])
            if 'step' in o:
                w.setSingleStep(o['step'])

        elif t == 'doubleSpin':
            w = QtGui.QDoubleSpinBox()
            if 'max' in o:
                w.setMaximum(o['max'])
            if 'min' in o:
                w.setMinimum(o['min'])                
            if 'value' in o:
                w.setValue(o['value'])
            if 'step' in o:
                w.setSingleStep(o['step'])

        elif t == 'check':
            w = QtGui.QCheckBox()
            if 'checked' in o:
                w.setChecked(o['checked'])
            if 'applyBox' in o:
                if o['applyBox'] is True:
                    w.setToolTip('When checked this node will process all incoming data.\n'
                                 'Therefore note that this will cause the program to behave\n'
                                 'slowly if you are constantly changing things around\n'
                                 'while keeping this checked.')

        elif t == 'radioBtn':
            w = QtGui.QRadioButton()
            if 'checked' in o:
                w.setChecked(o['checked'])

        elif t == 'combo':
            w = QComboBox()
            if 'values' in o.keys():
                for i in o['values']:
                    if i != '':
                        w.addItem(i)
            if 'items' in o.keys():
                w.setItems(o['items'])

        elif t == 'list_widget':
            w = ListWidget()
            if 'items' in o.keys():
                w.setItems(o['items'])
            if 'selection_mode' in o.keys():
                w.setSelectionMode(o['selection_mode'])

        elif t == 'lineEdit':
            # w = QtGui.QLineEdit()
            w = LineEdit()
            if 'placeHolder' in o:
                w.setPlaceholderText(o['placeHolder'])
            if 'text' in o:
                w.setText(o['text'])
            if 'readOnly' in o:
                w.setReadOnly(True)

        elif t == 'plainTextEdit':
            w = QtGui.QPlainTextEdit()
            if 'placeHolder' in o:
                w.setPlaceholderText(o['placeHolder'])
            if 'text' in o:
                w.setText(o['text'])

        elif t == 'kwargTextEdit':
            w = KwargPlainTextEdit()
            if 'placeHolder' in o:
                w.setPlaceholderText(o['placeHolder'])
            if 'text' in o:
                w.setText(o['text'])

        elif t == 'label':
            w = QtGui.QLabel()
            if 'text' in o:
                w.setText(o['text'])

        elif t == 'button':
            w = QtGui.QPushButton()
            if 'text' in o:
                w.setText(o['text'])
            if 'checkable' in o:
                w.setChecked(o['checkable'])

        elif t == 'color':
            w = ColorButton()
        else:
            raise Exception("Unknown widget type '%s'" % str(t))
        if 'toolTip' in o:
            w.setToolTip(o['toolTip'])
        w.setObjectName(k)
        l.addRow(k, w)
        if o.get('hidden', False):
            w.hide()
            label = l.labelForField(w)
            label.hide()
            
        ctrls[k] = w
        w.rowNum = row
        row += 1
    group = WidgetGroup(widget)
    return widget, group, ctrls


class CtrlNode(Node):
    """Abstract class for nodes with auto-generated control UI"""
    
    sigStateChanged = QtCore.Signal(object)
    
    def __init__(self, name, ui=None, terminals=None, **kwargs):
        """:param terminals: Dict containing terminal names and specifying whether they are input or output terminals"""

        if ui is None:
            if hasattr(self, 'uiTemplate'):
                ui = self.uiTemplate
            else:
                ui = []
        if terminals is None:
            terminals = {'In': {'io': 'in'}, 'Out': {'io': 'out', 'bypass': 'In'}}
        Node.__init__(self, name=name, terminals=terminals)
        
        self.ui, self.stateGroup, self.ctrls = generateUi(ui)
        self.stateGroup.sigChanged.connect(self.changed)
        self.count = 0
        self.t = None
        self._data_column = None

    def ctrlWidget(self):
        return self.ui
       
    def changed(self):
        self.update()
        self.sigStateChanged.emit(self)
        out = f"Processing node: {self.name()}\n"

        logger.info(out)

    def process(self, **kwargs):#In, display=True):
        In = kwargs['In']

        if In is None:
            return

        if type(In) is None:
            raise TypeError('Incoming transmission is None')

        if isinstance(In, Transmission):
            if In.df.empty:
                raise IndexError('The DataFrame of the incoming transmission is empty!')

        out = self.processData(In)
        return {'Out': out}
    
    def saveState(self):
        # self.changed()
        state = Node.saveState(self)
        state['ctrl'] = self.stateGroup.state()
        return state
    
    def restoreState(self, state):
        Node.restoreState(self, state)
        if self.stateGroup is not None:
            self.stateGroup.setState(state.get('ctrl', {}))
        # self.changed()
            
    def hideRow(self, name):
        w = self.ctrls[name]
        l = self.ui.layout().labelForField(w)
        w.hide()
        l.hide()
        
    def showRow(self, name):
        w = self.ctrls[name]
        l = self.ui.layout().labelForField(w)
        w.show()
        l.show()

    def set_data_column_combo_box(self):
        try:
            columns = self.t.df.columns
            self.ctrls['data_column'].setItems(organize_dataframe_columns(columns.to_list())[0])
            ix = self.ctrls['data_column'].findText(self.t.last_output)
            if ix > 0:
                self.ctrls['data_column'].setCurrentIndex(ix)
        except:
            pass

    @property
    def data_column(self):
        try:
            self._data_column = self.ctrls['data_column'].currentText()
            return self._data_column
        except:
            return None

    @data_column.setter
    def data_column(self, d):
        pass

    def apply_checked(self) -> bool:
        return self.ctrls['Apply'].isChecked()


class PlottingCtrlNode(CtrlNode):
    """Abstract class for CtrlNodes that can connect to plots."""
    
    def __init__(self, name, ui=None, terminals=None):
        #print "PlottingCtrlNode.__init__ called."
        CtrlNode.__init__(self, name, ui=ui, terminals=terminals)
        self.plotTerminal = self.addOutput('plot', optional=True)

    def connected(self, term, remote):
        CtrlNode.connected(self, term, remote)
        if term is not self.plotTerminal:
            return
        node = remote.node()
        node.sigPlotChanged.connect(self.connectToPlot)
        self.connectToPlot(node)    
        
    def disconnected(self, term, remote):
        CtrlNode.disconnected(self, term, remote)
        if term is not self.plotTerminal:
            return
        remote.node().sigPlotChanged.disconnect(self.connectToPlot)
        self.disconnectFromPlot(remote.node().getPlot())   
       
    def connectToPlot(self, node):
        """Define what happens when the node is connected to a plot"""
        raise Exception("Must be re-implemented in subclass")
    
    def disconnectFromPlot(self, plot):
        """Define what happens when the node is disconnected from a plot"""
        raise Exception("Must be re-implemented in subclass")

    def process(self, In, display=True):
        out = CtrlNode.process(self, In, display)
        out['plot'] = None
        return out

def metaArrayWrapper(fn):
    def newFn(self, data, *args, **kargs):
        if HAVE_METAARRAY and (hasattr(data, 'implements') and data.implements('MetaArray')):
            d1 = fn(self, data.view(np.ndarray), *args, **kargs)
            info = data.infoCopy()
            if d1.shape != data.shape:
                for i in range(data.ndim):
                    if 'values' in info[i]:
                        info[i]['values'] = info[i]['values'][:d1.shape[i]]
            return metaarray.MetaArray(d1, info=info)
        else:
            return fn(self, data, *args, **kargs)
    return newFn


def merge_transmissions(transmissions) -> list:
    if not len(transmissions) > 0:
        raise Exception('No incoming transmissions')

    transmissions_list = []

    for t in transmissions.items():
        t = t[1]
        if t is None:
            raise TypeError('One of your transmissions is None')
           # continue
        if type(t) is list:
            for i in range(len(t)):
                if t[i] is None:
                    raise TypeError('One of your transmissions is None')
                    # QtGui.QMessageBox.warning(None, 'None transmission', 'One of your transmissions is None')
                    # continue
                transmissions_list.append(t[i].copy())
            continue

        transmissions_list.append(t.copy())
    return transmissions_list


normalize_zero_one = lambda a: ((a - np.min(a)) / (np.max(a - np.min(a))))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# @author: kushal
#
# Chatzigeorgiou Group
# Sars International Centre for Marine Molecular Biology
#
# GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007


import numpy as np
from itertools import product
from ...common.utils import HdfTools
from ...common.configuration import HAS_TSLEARN
if HAS_TSLEARN:
    from tslearn.cycc import normalized_cc


def ncc_c(x: np.ndarray, y: np.ndarray) -> np.ndarray:
    """
    Must pass 1D array to both x and y

    :param x: Input array [x1, x2, x3, ... xn]
    :param y: Input array [y2, y2, x3, ... yn]

    :return:  Returns the normalized cross correlation function (as an array) of the two input vector arguments "x" and "y"
    :rtype: np.ndarray
    """
    cc = normalized_cc(x.reshape(-1, 1), y.reshape(-1, 1))
    return cc


def get_omega(x: np.ndarray = None, y: np.ndarray = None, cc: np.ndarray = None) -> int:
    """
    Must pass a 1D array to either both "x" and "y" or a cross-correlation function (as an array) to "cc"

    :param x: Input array [x1, x2, x3, ... xn]
    :param y: Input array [y2, y2, x3, ... yn]
    :param cc: cross-correlation function represented as an array [c1, c2, c3, ... cn]

    :return: index (x-axis position) of the global maxima of the cross-correlation function
    :rtype: np.ndarray
    """
    if cc is None:
        cc = ncc_c(x, y)
    w = np.argmax(cc)
    return w


def get_lag(x: np.ndarray = None, y: np.ndarray = None, cc: np.ndarray = None) -> float:
    """
    Must pass a 1D array to either both "x" and "y" or a cross-correlation function (as an array) to "cc"

    :param x: Input array [x1, x2, x3, ... xn]
    :param y: Input array [y2, y2, x3, ... yn]
    :param cc: cross-correlation function represented as a array [c1, c2, c3, ... cn]

    :return: Position of the maxima of the cross-correlation function with respect to middle point of the cross-correlation function
    :rtype: np.ndarray
    """
    if cc is None:
        w = get_omega(x, y)
        s = w - x.shape[0]
    else:
        w = get_omega(cc=cc)
        s = w - int(cc.shape[0] / 2)
    return float(-s)


def get_epsilon(x: np.ndarray = None, y: np.ndarray = None, cc: np.ndarray = None) -> float:
    """
    Must pass a 1D vector to either both "x" and "y" or a cross-correlation function to "cc"

    :param x: Input array [x1, x2, x3, ... xn]
    :param y: Input array [y2, y2, x3, ... yn]
    :param cc: cross-correlation function represented as an array [c1, c2, c3, ... cn]

    :return: Magnitude of the global maxima of the cross-correlationn function
    :rtype: np.ndarray
    """
    if cc is None:
        cc = ncc_c(x, y)
    e = np.max(cc)
    return e


def get_lag_matrix(curves: np.ndarray = None, ccs: np.ndarray = None) -> np.ndarray:
    """
    Get a 2D matrix of lags. Can pass either a 2D array of 1D curves or cross-correlations

    :param curves: 2D array of 1D curves
    :param ccs:    2D array of 1D cross-correlation functions represented by arrays

    :return: 2D matrix of lag values, shape is [n_curves, n_curves]
    :rtype: np.ndarray
    """
    if ccs is None:
        m = curves.shape[0]
        a = np.zeros((m, m))

        for i, j in product(*(range(m),) * 2):
            a[i, j] = get_lag(curves[i], curves[j])
        return a

    return _compute_from_ccs(ccs, get_lag)


def get_epsilon_matrix(curves: np.ndarray = None, ccs: np.ndarray = None) -> np.ndarray:
    """
    Get a 2D matrix of maximas. Can pass either a 2D array of 1D curves or cross-correlations

    :param curves: 2D array of 1D curves
    :param ccs:    2D array of 1D cross-correlation functions represented by arrays

    :return: 2D matrix of maxima values, shape is [n_curves, n_curves]
    :rtype: np.ndarray
    """
    if ccs is None:
        m = curves.shape[0]
        a = np.zeros((m, m))
        for i, j in product(*(range(m),) * 2):
            a[i, j] = get_epsilon(curves[i], curves[j])
        return a

    return _compute_from_ccs(ccs, get_epsilon)


def _compute_from_ccs(ccs: np.ndarray, func: callable) -> np.ndarray:
    m = ccs.shape[0]
    a = np.zeros(shape=(m, m))
    for i, j in product(*(range(m),) * 2):
        a[i, j] = func(cc=ccs[i, j, :])
    return a


class CC_Data:
    def __init__(self,
                 input_data: np.ndarray = None,
                 ccs: np.ndarray = None,
                 lag_matrix: np.ndarray = None,
                 epsilon_matrix: np.ndarray = None,
                 curve_uuids: np.ndarray = None,
                 labels: np.ndarray = None):
        """
        Object for organizing cross-correlation data

        types must be numpy.ndarray to be compatible with hdf5

        :param ccs:             array of cross-correlation functions, shape: [n_curves, n_curves, func_length]
        :type ccs:              np.ndarray
        :param lag_matrix:      the lag matrix, shape: [n_curves, n_curves]
        :type lag_matrix:       np.ndarray
        :param epsilon_matrix:  the maxima matrix, shape: [n_curves, n_curves]
        :type epsilon_matrix:   np.ndarray
        :param curve_uuids:     uuids (str representation) for each of the curves, length: n_curves
        :type curve_uuids:      np.ndarray
        :param labels:          labels for each curve, length: n_curves
        :type labels:           np.ndarray
        """
        self.input_data: np.ndarray = input_data
        self.ccs = ccs  #: array of cross-correlation functions
        self.lag_matrix = lag_matrix  #: lag matrix
        self.epsilon_matrix = epsilon_matrix  #: maxima matrix
        self.curve_uuids = curve_uuids  #: uuids for each curve
        self.labels = labels  #: labels for each curve

    def get_threshold_matrix(self, matrix_type: str, lag_thr: float, max_thr: float,
                             lag_thr_abs: bool = True) -> np.ndarray:
        """
        Get lag or maxima matrix with thresholds applied. Values outside the threshold are set to NaN

        :param matrix_type:     one of 'lag' or 'maxima'
        :param lag_thr:         lag threshold
        :param max_thr:         maxima threshold
        :param lag_thr_abs:     threshold with the absolute value of lag

        :return:    the requested matrix with the thresholds applied to it.
        :rtype: np.ndarray
        """
        # Get lag and maxima matrices
        l = self.lag_matrix
        m = self.epsilon_matrix

        if (l is None) or (m is None):
            raise ValueError('No lag or maxima data available')

        # The type of matrix to return
        if matrix_type == 'lag':
            a = np.copy(l)
        elif matrix_type == 'maxima':
            a = np.copy(m)
        else:
            raise ValueError('Argument "matrix_type" must be "lag" or "maxima"')

        # Set all values below both thresholds to nan
        if lag_thr_abs:
            a[np.abs(l) > lag_thr] = np.nan
        else:
            a[l > lag_thr] = np.nan

        a[m < max_thr] = np.nan

        return a

    @classmethod
    def from_dict(cls, d: dict):
        """Load data from a dict"""
        return cls(**d)

    def to_hdf5(self, path: str):
        """
        Save as an HDF5 file

        :param path: path to save the hdf5 file to, file must not exist.
        """

        HdfTools.save_dict(self.__dict__, path, 'cross_corr_data')

    @classmethod
    def from_hdf5(cls, path: str):
        """
        Load cross-correlation data from an hdf5 file

        :param path: path to the hdf5 file
        """
        d = HdfTools.load_dict(path, 'cross_corr_data')
        return cls(**d)


def compute_cc_data(curves: np.ndarray) -> CC_Data:
    """
    Compute cross-correlation data (cc functions, lag and maxima matrices)

    :param curves: input curves as a 2D array, shape is [n_samples, curve_size]

    :return:    cross correlation data for the input curves as a CC_Data instance
    :rtype: CC_Data
    """
    ccs = compute_ccs(curves)

    l = get_lag_matrix(ccs=ccs)
    e = get_epsilon_matrix(ccs=ccs)

    return CC_Data(curves, ccs, l, e)


def compute_ccs(a: np.ndarray) -> np.ndarray:
    """
    Compute cross-correlations between all 1D curves in a 2D input array

    :param a: 2D input array of 1D curves, shape is [n_samples, curve_size]

    :rtype: np.ndarray
    """
    n = a.shape[0]
    m = a.shape[1]
    out = np.zeros(shape=(n, n, (2 * m) - 1))

    for i, j in product(*(range(n),) * 2):
        out[i, j, :] = ncc_c(a[i], a[j])
    return out



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from PyQt5 import QtWidgets
import abc
from ...analysis import Transmission, organize_dataframe_columns
from ...common.qdialogs import *
from ...common import InheritDocs
from typing import *

#TODO: Implement base for all plot widgets


class _AbstractBasePlotWidget(metaclass=InheritDocs):
    @property
    @abc.abstractmethod
    def transmission(self) -> Transmission:
        """
        The input transmission

        :rtype: Transmission
        """
        pass

    @transmission.setter
    @abc.abstractmethod
    def transmission(self, transmission: Transmission):
        """
        Set the input Transmission

        :param transmission: Input transmission
        """

        pass

    @abc.abstractmethod
    def set_input(self, transmission: Transmission):
        """
        Set the input Transmission with data to plot

        :param transmission: Input transmission
        """

        pass

    @abc.abstractmethod
    def update_plot(self, *args, **kwargs):
        """Method that must must be used for updating the plot"""
        pass

    @abc.abstractmethod
    def set_update_live(self, b: bool):
        "Method to set if the plot updates with live input"
        pass

    @abc.abstractmethod
    def get_plot_opts(self) -> dict:
        """Package all necessary plot parameters that in combination with the transmission property are sufficient to restore the plot"""
        pass

    @abc.abstractmethod
    def set_plot_opts(self, opts: dict):
        """Set plot parameters from a dict in the format returned by get_plot_opts()"""
        pass

    @abc.abstractmethod
    def save_plot(self, *args):
        """Package plot data and plot parameters and save to a file.
        Must contain all the information that is necessary to restore the plot"""
        pass

    @abc.abstractmethod
    def open_plot(self, ptrn_path: str, proj_path: str) -> Union[Tuple[str, str], None]:
        """Open a plot file and restore the plot"""
        pass


class _MetaQtABC(QtWidgets.QWidget.__class__, _AbstractBasePlotWidget.__class__):
    pass


class BasePlotWidget(_AbstractBasePlotWidget, metaclass=_MetaQtABC):
    """
    Base for plot widgets.

    Subclasses must define the class attribute "drop_opts" which is used to store a list of JSON incompatible keys returned by the get_plot_opts() method
    """
    def __init__(self):
        super().__init__()
        self._transmission = None
        self.block_signals_list = []  #: List of QObjects included in dynamic signal blocking. Used for storing plot parameter widgets so that changing all of them quickly (like when restoring a plot) doesn't cause the plot to constantly update.
        self.previous_df_cols = []
        self.update_live = True

    def __init_subclass__(cls, **kwargs):
        if not hasattr(cls, 'drop_opts'):
            raise AttributeError('Must define class attribute "drop_opts"')

    @property
    def transmission(self) -> Transmission:
        if self._transmission is None:
            raise AttributeError("No input transmission has been set")
        return self._transmission

    @transmission.setter
    def transmission(self, transmission: Transmission):
        if not isinstance(transmission, Transmission):
            raise TypeError('Input must be an instance of Transmission')
        self._transmission = transmission

    def set_input(self, transmission: Transmission):
        self.transmission = transmission

        cols = set(self.transmission.df.columns)

        if set(self.previous_df_cols) != cols:
            dcols, ccols, ucols = organize_dataframe_columns(self.transmission.df.columns)

            cols = {'data_columns': dcols, 'categorical_columns': ccols, 'uuid_columns': ucols}

            self.fill_control_widget(**cols)

        self.previous_df_cols = cols

    def fill_control_widget(self, data_columns: list, categorical_columns: list, uuid_columns: list):
        """Method for filling the control widget(s) when inputs are set. Must be implemented in subclass"""
        raise NotImplementedError("""Must be implemented in subclass""")

    def update_plot(self, *args, **kwargs):
        """Must be implemented in subclass"""
        raise NotImplementedError('Must be implemented in subclass')

    def set_update_live(self, b: bool):
        """Must be implemented in subclass"""
        raise NotImplementedError('Must be implemented in subclass')

    def get_plot_opts(self, drop: bool) -> dict:
        """
        Must be implemented in subclass
        """
        raise NotImplementedError('Must be implemented in subclass')

    def set_plot_opts(self, opts: dict):
        """Must be implemented in subclass"""
        raise NotImplementedError('Must be implemented in subclass')

    @classmethod
    def signal_blocker(cls, func):
        """Use as a decorator. Block Qt signals from all QObjects instances in the block_signals_list"""
        def fn(self, *args, **kwds):
            restore_dict = dict.fromkeys(self.block_signals_list)

            for w in self.block_signals_list:
                restore_dict[w] = w.signalsBlocked()
                w.blockSignals(True)

            try:
                ret = func(self, *args, **kwds)
            finally:
                for w in self.block_signals_list:
                    w.blockSignals(restore_dict[w])

            return ret

        return fn

    @use_save_file_dialog('Save plot as', None, '.ptrn')
    def save_plot_dialog(self, path, *args):
        """Plot save dialog"""
        self.save_plot(path)

    @present_exceptions('Plot Save Error', 'The following error occurred while trying to save the plot.')
    def save_plot(self, path):
        """
        Save the plot as a Transmission in an HDF5 file. Plot parameters are stored as a JSON string within the HDF5 file.
        See Transmission.to_hdf5

        :param path: Path to save the file to. For easy identification use ".ptrn" extension.
        """
        # Drop the JSON incompatible data
        plot_state = self.get_plot_opts(drop=True)
        # if self.drop_opts is not None:
        #     for k in self.drop_opts:
        #         plot_state.pop(k)

        plot_state['type'] = self.__class__.__name__
        self.transmission.plot_state = plot_state
        self.transmission.to_hdf5(path)
        print(f"{self.__class__.__name__} plot saved to: {path}")

    @use_open_dir_dialog('Select Project Folder', None)
    @use_open_file_dialog('Choose plot file', None, ['*.ptrn'])
    def open_plot_dialog(self, filepath, dirpath, *args, **kwargs):
        """Open plot dialog"""
        self.open_plot(filepath, dirpath)

    @present_exceptions('Plot open error', 'The following error occurred while trying to open the plot')
    def open_plot(self, ptrn_path: str, proj_path: str) -> Union[Tuple[str, str], None]:
        """
        Open a plot saved by the save_plot() method

        :param ptrn_path: Path to the HDF5 Transmission file. By convention file extension is ".ptrn"
        :param proj_path: Project path of the associated plot data.
        """
        ptrn = Transmission.from_hdf5(ptrn_path)

        plot_state = ptrn.plot_state
        plot_type = plot_state['type']

        if not plot_type == self.__class__.__name__:
            raise TypeError(f'Wrong plot type\n\n'
                            f'The chosen file is not for this type of plot\n'
                            f'This file is for the following plot type: {plot_type}')

        ptrn.set_proj_path(proj_path)

        update_state = self.update_live

        self.set_update_live(False)
        self.set_input(ptrn)
        self.set_plot_opts(plot_state)
        self.set_update_live(update_state)

        self.update_plot()

        return ptrn_path, proj_path
